#!/bin/sh
#
# Copyright (C) 2004-2013 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of le-save-config
#
# le-save-config is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# Exit codes: ($? != 0)
# 1 -- RESERVED. Do not use; Sudo return code exclusive
# 2 -- Cancel on file selection
# 3 -- Need to be run as root
# 4 -- Error getting archive from cloud(internet)
# (OLD - DONT USE) 5 -- Error sending archive to cloud(internet)
# 6 -- Cancel on 'what to restore' dialog
# 7 -- Incompatible selected save file
# 8 -- Error to restore packages

# 10+x -- Where x is the return of le-save-config-bridge.sh

#EXIT 60 -- Ctrl+C
trap ctrl_c 2 3
ctrl_c () {
    [ -n "$!" ] && kill -s 9 "$!"
    exit 60
}

if [ $(id -u) != 0 ]; then
    echo "This script must be run as root!"
    gksu -S -l -m "É necessário ter permissão de superusuário. Digite a senha:" "sh /usr/sbin/le-save-config.sh"
    exit $?
fi

# Options:
OP1="Salvar configurações atuais na nuvem"
OP2="Salvar configurações atuais localmente"
OP3="Carregar configurações da nuvem"
OP4="Carregar configurações de arquivo local"
OP5="Ajuda"


mainScreen () {
    choice=$(
        zenity --list --hide-header \
              --height=240 \
              --width=350 \
              --radiolist \
              --text="Escolha a ação desejada:" \
              --title="RESTAURAR SISTEMA" \
              --column="" \
              --column="" \
               'FALSE' "$OP1" \
               'FALSE' "$OP2" \
               'FALSE' "$OP3" \
               'FALSE' "$OP4" \
               'FALSE' "$OP5"
    )
    f_ret=$?
    echo "$choice"
    return $f_ret # Used to know if user pressed Cancel(1) or Ok(0)
}


#================================================================================#
#=======================FUNCTIONS================================================#
#================================================================================#

#-Index-----------------#
#[Name] --> [warning]
#
#createSave
#sendToCloud
#progressPackages --> Only called by restorePackages
#restorePackages
#restoreWallpaper
#restoreProxy
#restoreUsers
#restoreFirefoxExtensions
#getFromCloud
#doRestore
#------------------------#

createSave () {

    local dir_of_new_save=$(pwd)

    mkdir -p le-save
    cd le-save

    # PROXY SAVING
    touch proxy

    # apt-get proxy
    if [ -f /etc/apt/apt.conf ]; then
        grep "^Acquire::http::proxy" /etc/apt/apt.conf >> proxy
    fi
    #http_proxy variable
    grep "^http_proxy=" /etc/environment >> proxy

    # FIREFOX SAVING
    mkdir firefox.prefs
    cd firefox.prefs
    local HOME_USER="${SUDO_USER}"
    if ls /home/"${USER}"/.mozilla/firefox/*.default/ 2>1 1>/dev/null; then #if user profile directory exists
        cp $HOME_USER/.mozilla/firefox/profiles.ini .
        # modify 'Path' string of the copied file to alunos.default
        sed -i "s/Path=.*/Path=alunos.default/g" profiles.ini #alunos.default is used in retore function
        # Firefox Proxy
        grep "network.proxy" $HOME_USER/.mozilla/firefox/*.default/prefs.js | sed '/network.proxy.backup/d' > firefox_proxy

        # Firefox Extensions
        cp -r $HOME_USER/.mozilla/firefox/*.default/extensions/ ./
        mkdir global_extensions
        cp -r /usr/lib/firefox/browser/extensions/* global_extensions/
    else
        echo "No '.default' found for user ${HOME_USER}. Skipping."
    fi
    cd ..

    # INSTALLED PACKAGES
    dpkg --get-selections > pkgs

    # USERS SAVE
    cat /etc/shadow | awk -F : '{if ($2!="*" && $2!="!") print $1}' > usrlist

    # WALLPAPER SAVE
    local wallpaper_name="$(sudo -u $SUDO_USER gsettings get org.gnome.desktop.background picture-uri | cut -b 9- | sed s/"'"/""/ )"
      #Sometimes url is with escape characters like %20 $25 %2f, this code resolve them (>using sh<)
        local wallpaper_decoded_name="$(echo -n "$wallpaper_name" | sed 's/+/ /g'| sed 's/\%0[dD]//g' | awk '/%/{while(match($0,/\%[0-9a-fA-F][0-9a-fA-F]/)) {$0=substr($0,1,RSTART-1)sprintf("%c",0+("0x"substr($0,RSTART+1,2)))substr($0,RSTART+3);}}{print}')"

    cp "$wallpaper_decoded_name" "wallpaper_$(echo $wallpaper_decoded_name | awk -F "/" '{print $NF}')"

    # COMPRESS SAVE
    cd ..
    tar cjf le-save.tar.bz2 le-save

    #Return - Need to return the new compressed file path
    #local ret=$(echo "$dir_of_new_save/le-save.tar")
    #return $ret
}

sendToCloud () {
    local file_to_send="$1"
    /usr/sbin/le-save-config-bridge.sh --upload "$file_to_send"
    return $?
}

progressPackages () {
    #------------------------WARNING---------------------------------------#
    #--This function's output should be the input for "zenity --progress"--#
    #----------------------------------------------------------------------#

    # Arguments: [$1-Packages to install(install)] [$2-To install packages(hold)] [$3-] #
    local all_packages="$1 $2"
    local number_of_packages="$(echo $all_packages | wc -w)"
  
    apt-get update

    echo "Begin"
    cont=0
    for each_pkg in $all_packages; do
        #Analysing package part
        apt_pkg_version="$(apt-cache show $each_pkg --no-all-versions | grep Version | cut -b 10- 2>/dev/null)"
        dpkg_pkg_version="$(dpkg -s $each_pkg | grep Version | cut -b 10- 2>/dev/null)"
        if [ -n "$apt_pkg_version" -a -n "$dpkg_pkg_version" -a "$apt_pkg_version" = "$dpkg_pkg_version" ] 
            then  #That means that newest version is already installed
               #Refresh bar and jump
               cont=$((cont + 1))
               echo $((cont*100/number_of_packages)) #Refresh bar 

               continue
        fi

        # Installing package part
        apt-get install $each_pkg --assume-yes
        local ret_apt="$?"
        if [ "$ret_apt" = "100" ]; then
            pkgs_error_list="$pkgs_error_list $each_pkg"
        elif [ "$ret_apt" != "0" ]; then
            echo "Package installing error(apt error code: $ret_apt)"
            zenity --error \
                --title="Ocorreu um erro" \
                --text="Ocorreu um erro ao instalar os pacotes (Código do Erro: ERRO-APT-$ret_apt)"
            exit 8
        fi

        #Refresh Progress Bar
        cont=$((cont + 1))
        echo $((cont*100/number_of_packages))
    done
}

restorePackages () {
    local all_pkgs_file="$1"
    local pkg_install="$(cat $all_pkgs_file | grep -w install$ | sed s/'install$'/''/)"
    local pkg_hold="$(cat $all_pkgs_file | grep -w hold$ | sed s/'hold$'/''/)"

    #Function progressPackages uses: pkgs_error_list
    pkgs_error_list=""
    progressPackages "$pkg_install" "$pkg_hold" | \
                            zenity --progress --no-cancel --auto-close --auto-kill \
                              --title="Restaurando Pacotes" --text="Este processo pode demorar."
    if [ "$pkg_hold" != "" ]; then
        #Put packages that were in hold before to hold
        for each in $pkg_hold; do
            echo "$each hold" | dpkg --set-selections
        done
    fi
    if [ "$pkgs_error_list" != "" ]; then
        zenity --warning \
          --title="Aviso" \
          --text="Os seguintes pacotes não foram encontrados no repositorio e portanto não foram instalados\n $(echo $pkgs_error_list | sed s/" "/"\n"/g)\n\nÉ necessário baixar diretamente do site do fornecedor" \
          --width="400"
    fi
}

restoreWallpaper () {
    local wall_file="$1"
    local restored_name="$(echo $wall_file | cut -b 11-)" #Cut "wallpaper_" prefix from name
    cp $wall_file "/usr/share/backgrounds/$restored_name"

    #For cron (makes background to refresh when set by gsettings)
    if [ -z "$DBUS_SESSION_BUS_ADDRESS" ] ; then
        local busDir=~/.dbus/session-bus
        export $(grep -h DBUS_SESSION_BUS_ADDRESS= $busDir/$(ls -1t $TMP | head -n 1))
    fi

    sudo -u $SUDO_USER gsettings set org.gnome.desktop.background picture-uri "file:///usr/share/backgrounds/$restored_name"
}

restoreProxy () {
    local PRO_FILE="$1"
    local aux_proxy

    # Restore http_proxy
    aux_proxy="$(grep http_proxy $PRO_FILE)"
    if [ "$aux_proxy" != "" ]; then
        sed -i '/http_proxy=/d' /etc/environment
        echo "$aux_proxy" >> /etc/environment
    fi

    # apt-get proxy
    aux_proxy="$(grep "Acquire::http::proxy" $PRO_FILE)"
    if [ "$aux_proxy" != "" ]; then
        if [ ! -f /etc/apt/apt.conf ]; then
           touch /etc/apt/apt.conf
        fi
        sed -i '/http_proxy=/d' /etc/apt/apt.conf
        echo $aux_proxy >> /etc/apt/apt.conf
    fi

    # Firefox Proxy
    cd firefox.prefs
    if [ ! -d /etc/le/home_alunos/.mozilla/firefox/alunos.default/ ]; then
        mkdir -p /etc/le/home_alunos/.mozilla/firefox/alunos.default/
        cp profiles.ini /etc/le/home_alunos/.mozilla/firefox/
    fi
    cp firefox_proxy /etc/le/home_alunos/.mozilla/firefox/alunos.default/prefs.js
    cp firefox_proxy $HOME/.mozilla/firefox/*.default/prefs.js
    cd ..
}

restoreUsers () {
    # First time users log in they need set a new pass
    local USR_FILE="$1"
    local all_users="$(cat $USR_FILE)"
    users_error=""
    for each_user in $all_users; do
        if [ "$(grep "$each_user:" /etc/passwd)" != "" ]; then
            # User already exist
            users_error=$(echo "$users_error $each_user")
        else
            useradd --create-home -d "/home/$each_user" $each_user
            echo "$each_user\n$each_user\n" | passwd $each_user
            passwd -e $each_user #expire password
        fi
    done
    if [ "$users_error" != "" ]; then
       zenity --warning --title="Aviso!"\
           --width=400 --height=200 \
           --text="Os seguintes usuários não foram restaurados pois já existem:\n$(echo "$users_error" | sed y/' '/'\n'/)" &
    fi
}

restoreFirefoxExtensions () {
    cd firefox.prefs

    if [ ! -d /etc/le/home_alunos/.mozilla/firefox/alunos.default/ ]; then
        mkdir -p /etc/le/home_alunos/.mozilla/firefox/alunos.default/
        cp profiles.ini /etc/le/home_alunos/.mozilla/firefox/
    fi
    cp -r global_extensions/* /usr/lib/firefox/browser/extensions/
    # Restore local extensions to students
    cp -r extensions/ /etc/le/home_alunos/.mozilla/firefox/alunos.default/
    # Restore local extensions to teacher
    cp -r extensions/ $HOME/.mozilla/firefox/alunos.default/

    cd ..
}

getFromCloud () {
    local save_from_cloud="$1"
    /usr/sbin/le-save-config-bridge.sh --download "$save_from_cloud"
    return $?
}

doRestore () {

    # Ask what the user want to restore
    restore_op1="Usuários"
    restore_op2="Programas"
    restore_op3="Proxy"
    restore_op4="Wallpaper"
    restore_op5="Plugins Firefox"
    restore_sel=$(
        zenity --list --hide-header \
           --height=210 \
           --width=350 \
           --checklist \
           --title="Restauração" \
           --text="Selecione o que deseja restaurar:" \
           --column="" \
           --column="" \
           'FALSE' "$restore_op1" \
           'FALSE' "$restore_op2" \
           'FALSE' "$restore_op3" \
           'FALSE' "$restore_op4" \
           'FALSE' "$restore_op5"
    )
    if [ "$?" = "1" ]; then # if cancel button was pressed exit
        exit 6
    fi

    # CALL RESTORE FUNCTIONS
    # Echos are for logging
    echo "Restaurando o sistema.."
    if [ "$(echo "$restore_sel" | grep "$restore_op2")" != "" ]; then
        echo "Restaurando os pacotes"
        restorePackages $PACKAGES_FILE
    fi
    if [ "$(echo "$restore_sel" | grep "$restore_op4")" != "" ]; then
        echo "Restaurando papel de parede"
        restoreWallpaper $WALLPAPER_FILE
        local show_wallpaper_message="true"
    fi
    if [ "$(echo "$restore_sel" | grep "$restore_op1")" != "" ]; then
        echo "Restaurando usuários"
        restoreUsers $PASSWD_FILE
    fi
    if [ "$(echo "$restore_sel" | grep "$restore_op5")" != "" ]; then
        echo "Restaurando Extensões de Firefox"
        restoreFirefoxExtensions \
            | zenity --progress --pulsate --auto-close --auto-kill --title="Restaurando Extensões" --text=""
    fi
    if [ "$(echo "$restore_sel" | grep "$restore_op3")" != "" ]; then
        echo "Restaurando configurações de proxy"
        restoreProxy $PROXY_FILE
    fi

    #[ "$show_wallpaper_message" = "true" ] && (zenity --info --title="Papel De Parede Restaurado" --text="O papel de parede foi restaurado e estará visível no próximo login" --height="150") &
    #echo "100" | zenity --progress --title="Restauração" --text="Restauração foi finalizada com sucesso"
}
#=================================================================================#
#======================END=FUNCTIONS==============================================#
#=================================================================================#

DIR_TMP="$(mktemp --directory)"
SAVE_DIR="$(pwd)"
cd $DIR_TMP

F_RET=0
while [ "$F_RET" -eq 0 ]; do
    CHOICE=$(mainScreen)
    F_RET=$?
    echo $CHOICE
    if [ "$F_RET" = "1" ]; then
        # User chose cancel option
        exit 0
    elif [ "$CHOICE" = "" ]; then
        # User chose OK
        # None of the options was selected
        zenity --info --text="Não foi selecionada nenhuma opção!"
    else
        # Exit the loop and continue the program
        break
    fi
done

case "$CHOICE" in
    "$OP1") # Save current configurations on the cloud
        le-save-config-bridge.sh -c
        bridge_ret="$?"
        case "${bridge_ret}" in
	    0 ) #Means there is a file. Ask if wish to overwrite.

               zenity --question --ok-label="Sim" --cancel-label="Não" 
                        --text="Já existe um arquivo de configuração para essa escola, deseja sobreecrevê-lo?"; 
               [ "$?" = "1" ] && exit 0 
	    ;;
            2 ) #permission denied

	        zenity --error --title="" --text=""
		exit 12 #(10+2)
	    ;;
	    9 ) #No config file exist yet
	    ;;
	esac

        (echo "Begin" ; createSave) | zenity --progress --pulsate --auto-close --auto-kill \
            --title="Construindo arquivo para enviar ao servidor" \
            --text="Este processo pode demorar alguns segundos."

        NEW_SAVE_FILE="$DIR_TMP/le-save.tar.bz2"

        ########To create a child is the way to preserve the $? of getFromCloud and have a progress bar 
        #//Create Progress Bar - Call sendToCloud - Treat return code
        (echo "GO"; while [ "1" = "1" ]; do sleep 0.5; done) | \
                zenity --progress --pulsate \
                --auto-close --auto-kill \
                --title="Enviando arquivo para o servidor" \
                --text="Este processo pode demorar alguns minutos." \
                --height="120" --width="500" &
        progress_pid="$!"
        sendToCloud $NEW_SAVE_FILE
        bridge_ret="$?"
        kill -9 $progress_pid
        if [ "$bridge_ret" != "0" ]; then
            zenity --error --title="Erro" --text="Erro ao enviar arquivo para o servidor (Código: $((10 + $bridge_ret)))"
            exit $((10 + $bridge_ret)) #Bridge error code exit(10+x) 
        fi
          
        rm -rf $NEW_SAVE_FILE
        ;;
    "$OP2") # Save current configuration on the local machine
        createSave
        NEW_SAVE_FILE="$DIR_TMP/le-save.tar.bz2"
        SAVE_TO="$(zenity --file-selection --filename="/home/$SUDO_USER/le-save.tar.bz2" --save --confirm-overwrite)"
        [ "$?" = "1" ] && exit 2
        if [ "$(echo $SAVE_TO | awk -F . '{print $NF}')" != "bz2" ]; then
            # Correct File Name
            SAVE_TO="${SAVE_TO}.bz2"
        fi
        cp "$NEW_SAVE_FILE" "$SAVE_TO"
        ;;
    "$OP3") # Get config from internet
        CONFIG_LOCAL_FILE="le-save.tar.bz2"
        ########To create a child is the way to preserve the $? of getFromCloud and have a progress bar
        (echo "GO"; while [ 1 ]; do sleep 0.5; done) | zenity --pulsate --progress \
                                           --auto-kill --auto-close \
                                           --title="Recebendo arquivo da Nuvem" \
                                           --text="Realizando download.. Este processo pode demorar alguns minutos" \
                                           --height="120" --width="500" &
        progress_pid="$!"
        getFromCloud $CONFIG_LOCAL_FILE
        ret_getFC="$?" #return from bridge
        kill -9 $progress_pid
        #######---------------------------
        case $ret_getFC in 
            2 ) #Error 2: INEP not found
                zenity --error --title="Erro" --text="Não foi possível encontrar o INEP da máquina."
                exit 12 #erro 10+2
            ;;
            10 ) #Error 10: Permission Denied
                #If here, INEP file should exist
                MAC_ADDR_MAQUINA="$(ifconfig | grep HWaddr | awk -F HWaddr '{print $NF}' | cut -b 2-)"
                INEP="$(cat /opt/seed_mec/SEED2/client/conf/inep)"
                zenity --error --title="Erro" --text="Erro: você não tem permissão para enviar ou receber o arquivo na nuvem. Esta máquina $MAC_ADDR_MAQUINA não faz parte do laboratório da escola cujo INEP é $INEP."
                exit 20 #erro 10+10
            ;;
            0 ) #Fine, continue
            ;;
	    * )
                zenity --error --title="Erro" --text="Ocorreu um erro ao tentar baixar arquivo da nuvem. (Código: $(( $ret_getFC + 10 )))"
                exit $(( 10 + $ret_getFC ))
            ;;
        esac

        if [ "$CONFIG_LOCAL_FILE" = "" ]; then
            exit 4
        fi

        # DECOMPRESS
        # Program hopes that the user selected an *.tar.bz2
        tar xjf $CONFIG_LOCAL_FILE # It will generate the directory 'le-save'
        cd le-save

        PACKAGES_FILE="pkgs"
        PASSWD_FILE="usrlist"
        PROXY_FILE="proxy"
        WALLPAPER_FILE="$(ls wallpaper_*)"
        doRestore
        ;;
    "$OP4") # Load configuration from a local file
        # SELECT FILE
        CONFIG_LOCAL_FILE=""
        while [ "$CONFIG_LOCAL_FILE" = "" ];
        do
            CONFIG_LOCAL_FILE="$(zenity --file-selection --filename="/home/$SUDO_USER/")"
            if [ "$?" = "1" ]; then
                zenity --question --title="" --text="Deseja cancelar a operação?"
                if [ "$?" = "0" ]; then
                    exit 2
                fi
            fi
        done

        # DECOMPRESS
        # Program hopes that the user selected an *.tar.bz2
        tar xjf "$CONFIG_LOCAL_FILE" #It will generate the directory le-save
        cd le-save

        PACKAGES_FILE="pkgs"
        PASSWD_FILE="usrlist"
        PROXY_FILE="proxy"
        WALLPAPER_FILE="$(ls wallpaper_*)"

        #Test files
        if [ ! -f pkgs -o ! -f proxy -o ! -f usrlist ]; then
            echo "Bad file"
            zenity --error --text="Arquivo selecionado incompatível" --title="Ocorreu um erro"
            exit 7
        fi

        doRestore
        ;;
    "$OP5") #Help option
        #If firefox is running by $SUDO_USER it must be called like that:
        su $SUDO_USER -c "sensible-browser /usr/local/share/doc/le-save-config/help.html"
        exit 0;
        ;;
esac

echo "DONE."

# Returning to original directory
cd "$SAVE_DIR"
rm -rf "$DIR_TMP"

# Successful Dialog
zenity --info --text="Operação realizada com sucesso" --title=""


exit 0
